#!/usr/bin/python
# -*- coding: UTF-8 -*-
#
#The MIT License
#
#Copyright (c) 2011
#
#Permission is hereby granted, free of charge, to any person obtaining a copy
#of this software and associated documentation files (the "Software"), to deal
#in the Software without restriction, including without limitation the rights
#to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#copies of the Software, and to permit persons to whom the Software is
#furnished to do so, subject to the following conditions:
#
#The above copyright notice and this permission notice shall be included in
#all copies or substantial portions of the Software.
#
#THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
#THE SOFTWARE.
#
#Authors:
#   Vili Auvinen (vili.k.auvinen@jyu.fi)
#   Olli Kauppinen (olli.kauppinen@jyu.fi)
#   Juho Tammela (juho.i.tammela@jyu.fi)

'''
This module contains common methods.

@author: Vili Auvinen, Olli Kauppinen, Juho Tammela
'''
def getTextContent (element):
    '''Return recursively all the text content from the given element. 
    This can be used in both, odt_inspector and docx_inspector.
    
    @param element: the element to be checked 
    
    @return: all text content from element and descendants
    '''
    elementsChildList = element.childNodes
    content = ''
    
    for childElement in elementsChildList:
        if childElement.nodeType == childElement.TEXT_NODE:
            content += childElement.nodeValue
        else:
            content += getTextContent(childElement)
    return content

def getAttributeContent(element):
    '''Return recursively all the attribute's content from the given element and it's children. 
    This can be used in both, odt_inspector and docx_inspector.
    
    @param element: the element to be checked 
    
    @return: all attribute's content from element and descendants
    '''
    content = ''
    
    
    for childElement in element.childNodes:
        if childElement.nodeType != childElement.TEXT_NODE:
            if childElement.hasAttributes():
                for i in range (0, len(childElement.attributes)):
                    content += childElement.attributes.item(i).value
        
            if childElement.hasChildNodes():
                content += getAttributeContent(childElement)
    return content

def checkStringFromContent(element,wantedString):  
    '''Check if given element contains given string.
    Encode to utf-8.
    
    @return: True if element contains given string, False if not.
    '''
    textContent = getTextContent(element)
    textContent.encode('utf-8')
    wantedString.encode('utf-8')
    if textContent.find(wantedString)==-1:
        return False
    else:
        return True
    
def checkIfEmailAddress (element):
    '''Check if element contains e-mail address.
    @-char is identifier.
    
    @return: e-mail address, None if element didn't contain e-mail.
    '''
    if checkStringFromContent(element,'@'):
        return getTextContent(element)
    return None

def getDescendants (element,elementList):
    '''Get elements all descendants.
    
    @param element: is the node which descendants have to find.    
    @param elementList: is empty list.
    
    @return: list of all descendants
    '''
    elementChildren = element.childNodes
    
    for element in elementChildren:
        elementList.append(element)
        if element.hasChildNodes():
            getDescendants(element,elementList)
    return elementList

def checkParentNode(element, parentTag):
    '''Check if element have given parentTag as parent
    
    @param element: the element to be checked 
    @param parentTag: is the tag which some parent have to be.
    
    @return: True if given element have parentTag as parent
    '''

    parent = element.parentNode
    
    while parent is not None:
        try:
            if parent.tagName == parentTag:
                return True
            else:
                parent = parent.parentNode
        except AttributeError:
            return False
    return False