#!/usr/bin/python
# -*- coding: UTF-8 -*-
#
#The MIT License
#
#Copyright (c) 2011
#
#Permission is hereby granted, free of charge, to any person obtaining a copy
#of this software and associated documentation files (the "Software"), to deal
#in the Software without restriction, including without limitation the rights
#to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#copies of the Software, and to permit persons to whom the Software is
#furnished to do so, subject to the following conditions:
#
#The above copyright notice and this permission notice shall be included in
#all copies or substantial portions of the Software.
#
#THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
#THE SOFTWARE.
#
#Authors:
#   Vili Auvinen (vili.k.auvinen@jyu.fi)
#   Olli Kauppinen (olli.kauppinen@jyu.fi)
#   Juho Tammela (juho.i.tammela@jyu.fi)

'''This module takes care of forwarding files to inspect to the proper inspector.

After the inspection compiles the feedback to be shown on the user interface.
@see: Function beginInspection().

@author: Vili Auvinen, Olli Kauppinen, Juho Tammela
'''


import zipfile
import word_processing
from cStringIO import StringIO
import xml.dom.minidom
from urllib import urlopen
from requirements import Requirements

def checkZipFiles(zipFile, fileName, feedback, requirements):
    """Checks if a file is a zip-file and forwards document files to the proper inspect-function.
    
    Recursively checks if files inside a zip-file are zip-files.
    If the file is a zip-file and it's in Microsoft Office or OpenOffice.org file format, gives the file forward.
    """
    fileExtension = getFileExtension(fileName)
    
    if fileExtension == "zip":
        try:
            zipDocument = zipfile.ZipFile(zipFile)
        except:
            feedback.append(stringToParagraph(fileName + " ei ole zip-tiedosto!"))
            return
        
        for f_info in zipDocument.infolist():
            data = zipDocument.read(f_info.filename)
            # If you need a file like object, wrap it in a cStringIO.
            fobj = StringIO(data)
            checkZipFiles(fobj, f_info.filename, feedback, requirements)
            
    elif(fileExtension  == "docx"):
        #print fileName       
        try:
            zipDocument = zipfile.ZipFile(zipFile)
        except:
            feedback.append(stringToParagraph(fileName + " ei ole zip-tiedosto!"))
            return
        results = word_processing.inspect(zipDocument, requirements, fileExtension)
        #printResults(results)
        feedback.append(resultsToDom(fileName, results))
    
    elif(fileExtension  == "pptx"):
        feedback.append(stringToParagraph("pptx-tiedostoja ei tueta vielä."))
    
    elif(fileExtension  == "xlsx"):
        feedback.append(stringToParagraph("xlsx-tiedostoja ei tueta vielä."))
    
    elif(fileExtension  == "odt"):
        #print fileName
        try:
            zipDocument = zipfile.ZipFile(zipFile)
        except:
            feedback.append(stringToParagraph(fileName + " ei ole zip-tiedosto!"))
            return
        results = word_processing.inspect(zipDocument, requirements, fileExtension)
        #printResults(results)
        feedback.append(resultsToDom(fileName, results))
    
    elif(fileExtension  == "odp"):
        feedback.append(stringToParagraph("odp-tiedostoja ei tueta vielä"))
    
    elif(fileExtension  == "ods"):
        feedback.append(stringToParagraph("ods-tiedostoja ei tueta vielä"))
    
    elif(fileExtension  == "docm"):
        try:
            zipDocument = zipfile.ZipFile(zipFile)
        except:
            feedback.append(stringToParagraph(fileName + " ei ole zip-tiedosto!"))
            return
        feedback.append(stringToParagraph(fileName + " on docm-tiedosto, käytä mieluummin docx-tiedostoja!"))
        results = word_processing.inspect(zipDocument, requirements, fileExtension)
        feedback.append(resultsToDom(fileName, results))
    elif(fileExtension  == "pptm"):
        feedback.append(stringToParagraph(fileName + " on pptm-tiedosto, käytä mieluummin pptx-tiedostoja. pptm-tiedostoja ei tueta vielä."))
    
    elif(fileExtension  == "ppsx"):
        feedback.append(stringToParagraph(fileName + " on ppsx-tiedosto, käytä mieluummin pptxx-tiedostoja. ppsx-tiedostoja ei tueta vielä."))
    
    else:
        feedback.append(stringToParagraph("Tiedostoa " + fileName + " ei osattu tarkastaa."))
        return
        
def stringToParagraph(s):
    '''Converts a string to html paragraph.'''
    impl = xml.dom.minidom.getDOMImplementation()
    Document = impl.createDocument(None, None, None)
    
    p = Document.createElement('p')
    txt1 = Document.createTextNode(s.decode('utf-8').encode('utf-8'))
    p.appendChild(txt1)
    
    return p.toxml()


def resultsToDom(fileName, results):
    '''Converts the results of an inspection with DOM to html.
    
    @param fileName: The name of the file where the results got.
    @param results: The results of the inspection.
    
    @return: Results as html.
    '''
    
    feedbackCount = 0
    
    translateDict = {
        'common':'Yleistiedot',
        'styles':'Tyylit',
        'objects':'Taulukot, kaaviot ja kuvat',
        'content':'Tekstiin tai sisältöön liittyvät virheet',
        'formatting':'Muotoilu',
        'structure': 'Rakenne',
        'numbers': 'Numerot ja tunnisteet'
        }
    
    impl = xml.dom.minidom.getDOMImplementation()
    Document = impl.createDocument(None, None, None)
    
    h21 = Document.createElement('h2')
    img1 = Document.createElement('img');
    img1.setAttribute('alt','Virhenäkymän laajennus.');
    img1.setAttribute('src','img/expand.png');
    h21.appendChild(img1)
    txt1 = Document.createTextNode(fileName.decode('utf-8').encode('utf-8'))
    h21.appendChild(txt1)
    
    ul1 = Document.createElement('ul')
#    ul1.setAttribute('id','filename');
    for key in results.keys():
        
        if len(results[key]) > 0:
            li1 = Document.createElement('li')
            li1.setAttribute("class", "1")
            ul1.appendChild(li1)
            txt6 = Document.createTextNode(translateDict[key])
            li1.appendChild(txt6)
        
        for result in results[key]:
            ul2 = Document.createElement('ul')
            li1.appendChild(ul2)
        
            if isinstance(result, dict):
                
                for key in result.keys():
                
                    li2 = Document.createElement('li')
                    li2.setAttribute("class", "2")
                    txt8 = Document.createTextNode(key.encode('utf-8'))
                    li2.appendChild(txt8)
                    ul2.appendChild(li2)
                    
                    ul3 = Document.createElement('ul')
                    li2.appendChild(ul3)
                    
                    for item in result[key]:
                        li3 = Document.createElement('li')
                        li3.setAttribute("class", "3")
                        ul3.appendChild(li3)
                        txt8 = Document.createTextNode(item.encode('utf-8', 'ignore'))
                        #txt8 = Document.createTextNode("%r" % item)
                        li3.appendChild(txt8)
                        feedbackCount += 1
            else:
                li2 = Document.createElement('li')
                li2.setAttribute("class", "2")
                ul2.appendChild(li2)
                txt8 = Document.createTextNode(result.encode('utf-8', 'ignore'))
                li2.appendChild(txt8)
                feedbackCount += 1
    
    summaryText = ""
    if feedbackCount == 0:
        summaryText = "Kaikki OK! :)"
    else:
        summaryText = 'Virheitä ' + str(feedbackCount) + ' kappaletta.'
    p1 = Document.createElement('p')
    txt3 = Document.createTextNode(summaryText.decode('utf-8').encode('utf-8'))
    p1.appendChild(txt3)
    
    div1 = Document.createElement('div')
    div1.setAttribute("class", "hideList")
    div1.appendChild(h21)
    div1.appendChild(p1)
    div1.appendChild(ul1)
    
    return div1.toxml()
        
def getFileExtension(fileName):
    '''Gets the file extension from a file name string.
    
    Splits the given string at char . and returns what is left on the right side of the last . char.
    
    @param fileName: File name as a string.
    
    @return: The file extension or the original string if character . is not found.
    '''
    splittedFileName = fileName.split(".")
    return str.lower(splittedFileName[-1])
            
def printResults(resultsDict):
    print "\nERRORS:"
    for feed in resultsDict.keys():
        print feed.upper(),
        if resultsDict[feed]:
            print ''
            for i in resultsDict[feed]:
                print "  -",i
        else:
            print '...OK'
            
def beginInspection(sentfile, filename, feedback, email):
    '''Begins the inspection of a file. Gets the requirements.xml using the email-address as a folder name.
    
    @param sentFile: The file to inspect, can be Microsoft Office docx, OpenOffice.org odt or a zip-file of these.
    @param filename: Name of the file.
    @param feedback: Empty list where the feedback of the inspection is stored.
    @param email: Email address of the user.
    
    @return: Error sring as a html paragraph if the requirements file is not found.
    '''
    #if len(sentfile.value) == 0:
    #    feedback.append("File not found!")
    #    return   '
    reqFileAddress = 'http://sovellusprojektit.it.jyu.fi/parsi/sovellus/requirements/'
    #FIXME: remove hard coded stuff later.
    try:
        if email == 'olli.kauppinen@jyu.fi':
            reqFilename = urlopen(reqFileAddress + email + '/requirements.xml')
            #reqFilename = 'requirements/'+email+'/requirements.xml'
            reqDom = xml.dom.minidom.parse(reqFilename)
            requirements = Requirements(reqDom)
        elif email == 'virheellinen':
            reqFilename = urlopen('http://sovellusprojektit.it.jyu.fi/parsi/sovellus/requirements/'+ 'virheellinen' + '/requirements.xml')
            #reqFilename = 'requirements/'+email+'/requirements.xml'
            reqDom = xml.dom.minidom.parse(reqFilename)
            requirements = Requirements(reqDom)
        else:
            reqFilename = urlopen(reqFileAddress + 'testi' + '/requirements.xml')
            reqDom = xml.dom.minidom.parse(reqFilename)
            requirements = Requirements(reqDom)
    except:
        return stringToParagraph('Vaatimustiedostoa ei löytynyt antamallesi sähköpostiosoitteelle.')
    
    checkZipFiles(sentfile, filename, feedback, requirements)
            
if __name__ == '__main__':
    feedback = []
#    )
#    print getFileExtension("testi.odt")
#    print getFileExtension("testi.DOCX")
#    print getFileExtension("testi.ODT")
#    print getFileExtension("testi.zip")
#    print getFileExtension("TESTI.ZIP")
#    print getFileExtension("testi")
#    print getFileExtension("TESTI")
#    

    #testExtension = getFileExtension("testi")

    #beginInspection("sampleFiles/odt/Yliopisto_opiskelu_korjattu.odt", "Yliopisto_opiskelu_korjattu.odt", feedback, 'olli.kauppinen@jyu.fi')
#    beginInspection("sampleFiles/docx/Teija_Holtta.docx", "Teija_Holtta.docx", feedback, 'olli.kauppinen@jyu.fi')
#    beginInspection("sampleFiles/docx/HarjoitustyoD.docx", "HarjoitustyoD.docx", feedback, 'olli.kauppinen@jyu.fi')
#    beginInspection("sampleFiles/docx/Harjoitustyo.docx", "Harjoitustyo.docx", feedback, 'olli.kauppinen@jyu.fi')
#    beginInspection("sampleFiles/testidokumentit_docx_odt.zip", "testidokumentit_docx_odt.zip", feedback, 'olli.kauppinen@jyu.fi')
#    beginInspection("sampleFiles/docx/docx_demo.docx", "docx_demo.docx", feedback, 'olli.kauppinen@jyu.fi')
    beginInspection("sampleFiles/docx/kaikki_oikein.docx", "kaikki_oikein.docx", feedback, 'olli.kauppinen@jyu.fi')
#    beginInspection("sampleFiles/docx/Aino_Anttila_Harjoitustyo.docx", "Aino_Anttila_Harjoitustyo.docx", feedback, 'olli.kauppinen@jyu.fi')
#    beginInspection('sampleFiles/docx/Harjoitustyo_elina_asikainen.docx', "Harjoitustyo_elina_asikainen.docx", feedback, 'olli.kauppinen@jyu.fi')
    #beginInspection("sampleFiles/docx/sivunumerointi_oikein_vili.docx", "sivunumerointi_oikein_vili.docx", feedback, 'olli.kauppinen@jyu.fi')
    #beginInspection("sampleFiles/docx/sivunumerointi_oikein.docx", "sivunumerointi_oikein.docx", feedback, 'olli.kauppinen@jyu.fi')
#    beginInspection('sampleFiles/docx/JoonaPohjonen.docx', "JoonaPohjonen.docx", feedback, 'olli.kauppinen@jyu.fi')

#    beginInspection('sampleFiles/docx/Harjoitustyo_elina_asikainen.docx', "Harjoitustyo_elina_asikainen.docx", feedback, 'virheellinen')
#    beginInspection('sampleFiles/docx/Aino_Anttila_Harjoitustyo.docx', "Aino_Anttila_Harjoitustyo.docx", feedback, 'virheellinen')
#    beginInspection('sampleFiles/docx/Aino_Anttila_Harjoitustyo.docx', "Aino_Anttila_Harjoitustyo.docx", feedback, 'virheellinen')
#    checkZipFiles("sampleFiles/odt/tekstinkasittely_ooo.odt")
#    checkZipFiles("sampleFiles/odt/calc_objektit.odt")
    #checkZipFiles("sampleFiles/docx/sivunumerointi_oikein_vili.docx", "sivunumerointi_oikein_vili.docx", feedback)
#    checkZipFiles("sampleFiles/odt/ume_ref_grad_windsor_01.odt", "ume_ref_grad_windsor_01.odt", feedback)
#    checkZipFiles("sampleFiles/docx/220311B.docx", "220311B.docx", feedback)    

    #for item in feedback:
    #    print item
    