#!/usr/bin/python
# -*- coding: UTF-8 -*-
#
#The MIT License
#
#Copyright (c) 2011
#
#Permission is hereby granted, free of charge, to any person obtaining a copy
#of this software and associated documentation files (the "Software"), to deal
#in the Software without restriction, including without limitation the rights
#to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#copies of the Software, and to permit persons to whom the Software is
#furnished to do so, subject to the following conditions:
#
#The above copyright notice and this permission notice shall be included in
#all copies or substantial portions of the Software.
#
#THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
#THE SOFTWARE.
#
#Authors:
#   Vili Auvinen (vili.k.auvinen@jyu.fi)
#   Olli Kauppinen (olli.kauppinen@jyu.fi)
#   Juho Tammela (juho.i.tammela@jyu.fi)

'''
This module provides the methods for inspecting odt files.

@author: Vili Auvinen, Juho Tammela, Olli Kauppinen
'''

import conversions
import string
import ooo_meta_inspector
import common_methods

def _getStyleElementByStyleName (documentDict, styleName):
    '''Get style element by given style name.
    Search first from content.xml if doesn't find then searches from style.xml. 
    
    @return: style element if style exists if doesn't return None.   
    '''
    styleElements = documentDict['content.xml'].getElementsByTagName('style:style')
    for element in styleElements:
        if element.getAttribute('style:name') == styleName:
            return element
                
    styleElements = documentDict['styles.xml'].getElementsByTagName('style:style')
    styleElements += documentDict['styles.xml'].getElementsByTagName('text:list-style')
    for element in styleElements:
        if element.getAttribute('style:name') == styleName:
            return element
    return None  


def _getStyleElementByDisplayName (documentDict, styleName):
    '''Get style element by given style name.
    Searches first from content.xml if doesn't find then searches from style.xml.
     
    @return: element if style display name exists if doesn't return None. 
    
    Example:
    <style:style style:name="Text_20_body" style:display-name="Text body">
    <style:style style:name="tutkielma">
    
    Display name --> style name
    " "-->"_20_"
    "_"-->"_5f_"    
    '''
    styleElements = documentDict['content.xml'].getElementsByTagName('style:style')
    for element in styleElements:
        if element.getAttribute('style:display-name').lower() == styleName.lower():
            return element
                
    styleElements = documentDict['styles.xml'].getElementsByTagName('style:style')
    for element in styleElements:
        if element.getAttribute('style:display-name').lower() == styleName.lower():
            return element
    return None 

def _getStyleDisplayNameByStyleName (documentDict, styleName):
    '''Get style display name by given style name.
    Uses _getStyleElementByStyleName-method to find the style.
    If style don't have display-name -attribute then display name is 
    just style name. 
    
    @return: style display name.
    
    Example:
    <style:style style:name="Text_20_body" style:display-name="Text body">
    <style:style style:name="tutkielma">
    
    Display name --> style name
    " "-->"_20_"
    "_"-->"_5f_"       
    '''
    
    element = _getStyleElementByStyleName(documentDict, styleName)
    if element.hasAttribute('style:display-name'):
        return element.getAttribute('style:display-name')
    else:
        return styleName
    
def _getMasterPageStyleElement (documentDict, masterPageStyleName):
    '''Get master page style element by given master page style name.
    
    @return: master page element.    
    '''
    masterPageStyles = documentDict['styles.xml'].getElementsByTagName ('style:master-page')
    for element in masterPageStyles:        
        if masterPageStyleName == element.getAttribute('style:name'):
            return element
    return None
        
def _getPageLayoutElement(documentDict, pageLayoutName):
    '''Get page layout element by given page layout name.
    
    @return: page layout.
    '''
    pageLayoutElements = documentDict['styles.xml'].getElementsByTagName ('style:page-layout')
    for element in pageLayoutElements:
        if pageLayoutName == element.getAttribute('style:name'):
            return element
    return None

        
def getPageMarginals(documentDict):
    '''Get page marginals.
    Notice only used master pages in document.
    
    @return: page marginals. If marginals are different between used pages,
    then return false.
    
    @see: convertCmOrInDictToString
    '''
    margins = {'top': None, 'bottom': None, 'right': None, 'left': None}
    usedMasterPages = _getUsedMasterPageElements(documentDict) 
    
    for element in usedMasterPages:
        pageLayoutName = element.getAttribute('style:page-layout-name')
        layoutElement = _getPageLayoutElement(documentDict, pageLayoutName)
        for key in margins.keys():            
            margin = layoutElement.getElementsByTagName('style:page-layout-properties')[0].getAttribute('fo:margin-' + key)
            
            if margins [key] is None:
                margins [key] = margin
            elif margins [key] != margin:   
                return False
            
                
            
    return conversions.convertCmOrInDictToString(margins)

def getPageSize(documentDict):
    '''Get page size.  
    
    @return: converted page size. If size is different between used pages,
    then return False. 
    
    @see: convertCmOrInDictToString
    '''
    pageSize = {'height': None, 'width': None }
    
    usedMasterPages = _getUsedMasterPageElements(documentDict)        
    
    for element in usedMasterPages:
        pageLayoutName = element.getAttribute('style:page-layout-name')
        layoutElement = _getPageLayoutElement(documentDict, pageLayoutName)
        for key in pageSize.keys():            
            size = layoutElement.getElementsByTagName('style:page-layout-properties')[0].getAttribute('fo:page-' + key)
            
            if pageSize [key] is None:
                pageSize [key] = size
            if pageSize [key] != size:
                return False
            
    return conversions.convertCmOrInDictToString(pageSize)

def _getUsedMasterPageElements (documentDict):
    ''' Get all master page elements which is used in document.
    'Standard' master page style used if there is no other definitions. 
    
    @return: list of used master page elements.
    '''
    usedMasterPageElements = []
    usedMasterPageDict = _getAllStyleNamesWithDifferentMasterPage(documentDict)
    for masterPageName in usedMasterPageDict.values():        
        masterPageElement = _getMasterPageStyleElement(documentDict, masterPageName)
        usedMasterPageElements.append(masterPageElement)
        
    if len(usedMasterPageElements) == 0:
        usedMasterPageElements.append(_getMasterPageStyleElement(documentDict, 'Standard'))
    return usedMasterPageElements    
    
def _getDefaultStyleElement(documentDict, styleFamily):
    '''Get default style element by given style family.
    
    @param styleFamily: tells which default style is wanted.
    
    @return: default style element.
    
    Style family:
    paragraph, graphic, table and table-row.
    
    Every style is based on style family. 
    <style:style style:name="Heading_20_1" style:display-name="Heading 1" style:family="paragraph">
    '''
    defaultStyleElements = documentDict['styles.xml'].getElementsByTagName('style:default-style')
    for element in defaultStyleElements:
        if element.getAttribute ('style:family') == styleFamily:
            return element
    return None


def _getMasterPageStyleName(documentDict, styleElement):
    '''Get master page style name by given style element.
    
    @return: master page name, if master page is '' then return 'Standard'.
    '''
    masterPageName = styleElement.getAttribute ('style:master-page-name')
    if masterPageName == '':
        return 'Standard'
    else:
        return masterPageName

def checkEmptyParagraphs(documentDict):
    '''Check empty paragraphs from document.
    getDocumentPararaphs method gets all paragraphs to check for.
    There is legal empty paragraph after table of content and in page break elements.   
     
    @return: False if there isn't empty paragraphs if there is then return number of empty paragraphs.
    '''
    pageBreakStyles = _getPageBreakStyleNames(documentDict)
    paragraphs = _getDocumentParagraphs(documentDict)
    emptyParagraphs = 0
    for element in paragraphs:
        if not element.childNodes and element.previousSibling.tagName != 'text:table-of-content' and element.getAttribute('text:style-name')not in set(pageBreakStyles):
            emptyParagraphs = emptyParagraphs + 1
    
    if emptyParagraphs == 0:
        return False
    return emptyParagraphs

def checkDoubleWhitespaces(documentDict):
    '''Check double spaces.
    Check if document have text:s-tag.
    
    @return: amount of double spaces.
    
    XML:
    <text:s text:c="2"/> --> 3 spaces
    <text:s/> --> 2 spaces
    '''
    doubleSpaces = documentDict['content.xml'].getElementsByTagName('text:s')
    
    if len(doubleSpaces) == 0:
        return False
    
    return len(doubleSpaces)    

def checkTabs(documentDict):
    '''Check tabs from document.
    getDocumentPararaphs method gets all paragraphs to check for.
    
    @return: False if there isn't tabs if there is then return number of tabs.
    '''
    paragraphs = _getDocumentParagraphs(documentDict)
    tabs = 0
    for element in paragraphs:
        elementListLength = len(element.getElementsByTagName('text:tab'))
        if  elementListLength != 0:
            tabs = tabs + elementListLength
    if tabs == 0:
        return False

    return tabs

def checkAsterisk(documentDict):
    '''Check asterisk from document.
    getDocumentPararaphs method gets all paragraphs to check for.
    
    @return: False if there isn't asterisks if there is then return number of asterisks.
    '''
    paragraphs = _getDocumentParagraphs(documentDict)
    asterisk = 0
    for element in paragraphs:
        if common_methods.checkStringFromContent(element, '*') is True:
            asterisk += 1
    if asterisk == 0:
        return False
    return asterisk

def _getDocumentParagraphs(documentDict):
    '''Get all paragraphs from document.
    All text-p(text paragraphs) and text-h (headings) elements.
    Used in checkTabs and checkEmptyParagraphs.
    
    @return: list of used elements.
    '''
    #FIXME: must append more paragraphs (like text:list) and deeper (all descendants)
    elementList = []
    officeTextElement = documentDict['content.xml'].getElementsByTagName ('office:text')[0]
    elements = officeTextElement.childNodes
    for element in elements:
        if element.nodeName == 'text:p' or element.nodeName == 'text:h':
            elementList.append(element)
    return elementList            
         
def _getListOfUsedStyleElements (documentDict):
    '''Get all used style elements.
    Content.xml->office:body-element contains used styles.
    
    @return: element list of used styles.
    '''
    
    usedStyleElements = []   
    
    bodyElements = common_methods.getDescendants(documentDict['content.xml'].getElementsByTagName('office:body')[0], [])
    
    for element in bodyElements:
        if element.nodeType is not element.TEXT_NODE and element.hasAttribute('text:style-name'):
            if len(common_methods.getTextContent(element)) != 0:
                usedStyleElements.append(element)
            
    return usedStyleElements

def _getListOfUsedStyleNames (documentDict):
    '''Get all used style names.
    Gets PX-styles (X = int value) parent style.
    (P1->Heading_20_1)
    
    @return: list of all used style names.
    '''
    usedStyleNames = []
    for element in _getListOfUsedStyleElements (documentDict):
        styleName = element.getAttribute('text:style-name')
        if styleName [0] == 'P' and styleName[1].isdigit():
            styleElement = _getStyleElementByStyleName(documentDict, styleName)
            if styleElement.hasAttribute('style:parent-style-name'): # if P-style doesn't have parent-style-name, it's not real style.
                usedStyleNames.append(styleElement.getAttribute('style:parent-style-name'))
        else:
            usedStyleNames.append(element.getAttribute('text:style-name'))
    
    usedStyleNames = list(set(usedStyleNames)) #deletes duplicates from list
    usedStyleNames.sort()

    return usedStyleNames   
    
def _checkPageBreakStyleElement (styleElement):
    '''Check if given style element does page break.
    
    @return: style element if style breaks the page return False if not. 
    '''
    
    hasParagraphProperties = styleElement.getElementsByTagName('style:paragraph-properties')
    if hasParagraphProperties:
        if hasParagraphProperties[0].getAttribute('fo:break-before') == 'page':
            return styleElement
    return False 

def _getPageBreakStyleNames(documentDict):
    '''Get names of styles which does page break.
    
    @return: list of page break style names.    
    '''
    pageBreakStylesNames = []
    styleElements = documentDict['content.xml'].getElementsByTagName('style:style')
    styleElements += documentDict['styles.xml'].getElementsByTagName('style:style')
    for element in styleElements:
        pageBreak = _checkPageBreakStyleElement(element)
        if pageBreak is not False:
            pageBreakStylesNames.append(element.getAttribute('style:name'))
      
    return pageBreakStylesNames

def _getAllStyleNamesWithDifferentMasterPage(documentDict):
    '''Get all style names which changes the master page.
    Master page will change when style has master-page-name -attribute
    and attribute value is not empty (""). If is empty ("") then master page is standard and
    if has no attribute the it same as previous master page.
    
    @param masterPageDict: key is style name and value is master page name.
    
    @return: dictionary of styles which changes master page.    
    '''
    styleElements = documentDict['content.xml'].getElementsByTagName('style:style')
    styleElements += documentDict['styles.xml'].getElementsByTagName('style:style')
    
    masterPageDict = {}
    
    for element in styleElements:
        if element.getAttribute('style:master-page-name') != '':
            masterPageDict[element.getAttribute('style:name')] = element.getAttribute('style:master-page-name')
    return masterPageDict
    
def _getSectionBreakElements(documentDict):
    '''Get section break elements from document. 
    Finds elements (text,list, heading...) which chance section. 
    
    @return: list of elements which changes section.  
    '''
    usedStyleElementsInDocument = _getListOfUsedStyleElements(documentDict)
    masterPagesDict = _getAllStyleNamesWithDifferentMasterPage(documentDict)
    sectionBreakElements = []
    for element in usedStyleElementsInDocument:
        if masterPagesDict.has_key(element.getAttribute('text:style-name')):
            sectionBreakElements.append(element)
    return sectionBreakElements        

def _getTOC(documentDict):
    '''Get table of content.
    Each TOC entry is own entry in tocList.
    
    @return: list of elements of table of content.     
    '''
    tocList = []
    if checkTOC(documentDict) is True:
        toc = documentDict['content.xml'].getElementsByTagName('text:table-of-content')
        tocTextElements = toc[0].getElementsByTagName ('text:p')
        for element in tocTextElements:
            if element.parentNode.nodeName == 'text:index-title':
                tocTitle = common_methods.getTextContent(element)
            else:
                tocList.append(common_methods.getTextContent(element))
        return tocList
   
def checkTocContent(documentDict):
    '''Compare document headings to TOC entries. 
    
    @return: True if all entries matches otherwise return error message.
    '''
    #FIXME: add better error messages (not False)
    tocList = _getTOC(documentDict)
    headingList = _getHeadingList(documentDict)['headings']

    if len(tocList) != len(headingList):
        return False#"Number of document headings doesn't match with number of TOC entries "
    i = 0 
    while i < len(tocList):
        if string.find (tocList[i], headingList[i]) == -1:
            return False #headingList[i-1]+" -heading doesn't exist in table of content"
        i += 1
    return True
    
    
        
def checkTOC(documentDict):
    '''Check if document contains table of contents.
    
    @return: True if there is table of content, return False if not. 
    '''
    toc = documentDict['content.xml'].getElementsByTagName('text:table-of-content')
    if len(toc) == 0:
        return False
    else:
        return True
    
def checkIndex(documentDict):
    '''Check if document have alphabetical index.
    
    @return: True if alphabetical index exists otherwise return False.
    '''
    aIndex = documentDict['content.xml'].getElementsByTagName('text:alphabetical-index')
    if len(aIndex) == 0: return False    
    return True

def _getIndexContentFromDocument (documentDict):
    '''Get marked alphabetical index entries from document.
    
    @return: content list of alphabetical index entries.
    '''
    indexContentList = []
    indexMarks = documentDict['content.xml'].getElementsByTagName ('text:alphabetical-index-mark-start')
    for element in indexMarks:
        indexContentList.append (element.nextSibling.nodeValue)
    return indexContentList

def _getIndexContent(documentDict):
    '''Get alphabetical index content.
    Each alphabetical index entry is own entry in list.
    
    @return: list of alphabetical index content.
    '''
    alphabeticalIndexList = [] 
    if checkIndex(documentDict) is True:
        aIndex = documentDict['content.xml'].getElementsByTagName('text:alphabetical-index')[0]
        aIndexTextElements = aIndex.getElementsByTagName('text:p')
        for element in aIndexTextElements:
            alphabeticalIndexList.append(common_methods.getTextContent(element))
    return alphabeticalIndexList       

def checkIndexContent(documentDict):
    '''Compare document marked texts to alphabetical index entries.
     
    @return: True if all entries matches otherwise return error code.
    '''
    indexList = _getIndexContent(documentDict)
    markedList = _getIndexContentFromDocument(documentDict)
    if len(indexList) == 0:
        return '3' # errorcode 3 = Index is empty
    
    for markedItem in markedList:
        for indexItem in indexList:
            found = False
            if string.find (indexItem, markedItem) != -1:
                found = True
                break  
        if found is False:
            return '4' #errorcode 4 = marked item not found from index
                   
    return True

def _getHeadingList(documentDict):
    '''Get all headings from document and used outline level.
    Each heading is own entry in list.
    
    @return: dictionary ['headings'] contains list of headings and ['level'] contains value of highest used heading outline level.
    '''
    headingList = []
    headingOutlineLevel = 0
    headings = documentDict['content.xml'].getElementsByTagName('text:h')
    for element in headings:
        heading = common_methods.getTextContent(element)
        headingList.append(heading)
        if headingOutlineLevel < element.getAttribute('text:outline-level'):
            headingOutlineLevel = element.getAttribute('text:outline-level')
    return {'headings':headingList, 'level':headingOutlineLevel}
    
def checkTable (documentDict):
    '''Check if document has a table.
    
    @return: True if there is table and False if not.
    '''
    table = documentDict['content.xml'].getElementsByTagName('table:table')
    if len(table) == 0:
        return True # "There is no table at all."
    else:
        return True
    

def _getTableDict (documentDict):
    '''Get tables in dictionary.
    Every table is own entry in tablesDict (key = table1,table2...)
    Every tableDict has table's cell address as key (A1,A2...) and cell value as dictionary's value.
    
    @return: dictionary of table dictionaries.
    '''
    tablesDict={}
    tableNumber = 0
    if checkTable (documentDict) is True:
        tableElements = documentDict['content.xml'].getElementsByTagName('table:table')
        for tableElement in tableElements:
            tableDict = {}
            
            tableRowElements = tableElement.getElementsByTagName('table:table-row')
            rowIndex = 0
            for row in tableRowElements:
                rowIndex += 1
                rowCellElements = row.getElementsByTagName('table:table-cell')
                columnIndex = ord('A') - 1
                for cell in rowCellElements:
                    columnIndex += 1
                    index = chr(columnIndex) + str(rowIndex)
                    tableDict[index] = common_methods.getTextContent(cell)
            tableNumber +=1
            tablesDict['table'+str(tableNumber)]=tableDict
        return tablesDict
    return False #"There is no table at all"
                
            

def checkPageNumberFromFooterAndHeader(documentDict, masterPageElement, element):
    '''Check page number format by given element and master page element.
    
    @param masterPageElement: master page element to look for.
    @param element: footer or header element. 
   
    @return: number format if it exists otherwise return False.
    
    Number format is optionally in element (footer or header). If number format
    is not in element then page-layout defines number format.
    '''
    pageLayoutElement = _getPageLayoutElement(documentDict, masterPageElement.getAttribute('style:page-layout-name'))
    pageNumberElements = element.getElementsByTagName('text:page-number')
    if pageNumberElements:
        if pageNumberElements[0].hasAttribute('style:num-format'):
            numFormat = pageNumberElements[0].getAttribute('style:num-format')
        else:
            numFormat = pageLayoutElement.getElementsByTagName('style:page-layout-properties')[0].getAttribute('style:num-format')
        return numFormat
    
    return False    
    
def getAuthorAndPageNumberFormat(documentDict, masterPageElement): 
    '''Get author and number format from header and footer.
    
    @param masterPageElement: master page element to look for.
    
    @return: dictionary which contains author and page number format.
    '''      

    meta = ooo_meta_inspector.getMeta(documentDict)
    footer = masterPageElement.getElementsByTagName ('style:footer')
    header = masterPageElement.getElementsByTagName ('style:header')  
    
    authorAndNumberDict = {'headerPageNumber':None, 'headerAuthor': None, 'footerPageNumber': None, 'footerAuthor': None}
        
    if footer:  
        if checkPageNumberFromFooterAndHeader(documentDict, masterPageElement, footer[0]) is not False:
            authorAndNumberDict['footerPageNumber'] = checkPageNumberFromFooterAndHeader(documentDict, masterPageElement, footer[0])
        if common_methods.checkStringFromContent(footer[0], meta['dc:creator']):
            authorAndNumberDict['footerAuthor'] = meta['dc:creator']

    if header:
        if checkPageNumberFromFooterAndHeader(documentDict, masterPageElement, header[0]) is not False:
            authorAndNumberDict['headerPageNumber'] = checkPageNumberFromFooterAndHeader(documentDict, masterPageElement, header[0])
            
        if common_methods.checkStringFromContent(header[0], meta['dc:creator']):
            authorAndNumberDict['headerAuthor'] = meta['dc:creator']

    return authorAndNumberDict       
    

def checkHeadingNumbering(documentDict, errorIdsAndPositions):
    '''Check outline style.
    level is highest used headings outline level. Normally Heading 1 --> 1, Heading 2 --> 2
    
    @return: True if ok, False if not.
    '''
    
    '''XML:
    <text:outline-style style:name="Outline">
        <text:outline-level-style text:level="1" style:num-format="1">
            <style:list-level-properties text:list-level-position-and-space-mode="label-alignment">
                <style:list-level-label-alignment text:label-followed-by="listtab" text:list-tab-stop-position="0.762cm" fo:text-indent="-0.762cm" fo:margin-left="0.762cm"/>
            </style:list-level-properties>
        </text:outline-level-style>

        <text:outline-level-style text:level="2" style:num-format="1" text:display-levels="2">
            <style:list-level-properties text:list-level-position-and-space-mode="label-alignment">
                <style:list-level-label-alignment text:label-followed-by="listtab" text:list-tab-stop-position="1.016cm" fo:text-indent="-1.016cm" fo:margin-left="1.016cm"/>
            </style:list-level-properties>
        </text:outline-level-style>

        <text:outline-level-style text:level="3" style:num-format="">
            <style:list-level-properties text:list-level-position-and-space-mode="label-alignment">
                <style:list-level-label-alignment text:label-followed-by="listtab" text:list-tab-stop-position="1.27cm" fo:text-indent="-1.27cm" fo:margin-left="1.27cm"/>
            </style:list-level-properties>
        </text:outline-level-style>   
        .
        .
    </text:outline-style>
    '''
    
    outlineStyleElements = documentDict['styles.xml'].getElementsByTagName('text:outline-style')
    
    if len(outlineStyleElements) == 0:
        outlineStyleElement = _getStyleElementByStyleName(documentDict, 'Outline')
        outlineLevels = outlineStyleElement.getElementsByTagName('text:list-level-style-number')
    else:
        outlineStyleElement = outlineStyleElements[0]
        outlineLevels = outlineStyleElement.getElementsByTagName('text:outline-level-style')
    
    
    level = _getHeadingList(documentDict)['level']
    index = 0
    for element in outlineLevels:  
        index = index + 1
        if index > int(level):
            return True         
        if int(element.getAttribute('text:level')) == index and element.getAttribute('style:num-format') != "":
            "ok"#index"-level heading numbering is correctly made"
        else:
            return False#"numbering is not correct on "+ str(level)+ "-level"


def _getImagePaths(documentDict):
    '''Get image paths.
    Checks if document have an image.
    Images is in picture folder.
    
    @return: images paths in list if founds an images otherwise return False.
    '''
    imagePathList = []
    if checkImages(documentDict) is True:
        imageElements = documentDict['content.xml'].getElementsByTagName ('draw:image')
        for element in imageElements:
            imagePath = element.getAttribute ('xlink:href')
            imagePathList.append(imagePath)
    
    if len(imagePathList)==0:
        return False   
    return imagePathList  

        
def checkImages(documentDict):
    '''Check if document contains an image.
    
    @return: true if there is an image otherwise return False. 
    '''
    imageElements = documentDict['content.xml'].getElementsByTagName ('draw:image')
    if len(imageElements) == 0:
        return False
    else:
        return True
    
def checkList(documentDict):
    '''Check if document contains list.
    
    @return: true if there is a list, return False if not. 
    '''
    listElements = documentDict['content.xml'].getElementsByTagName ('text:list')
    if len(listElements) == 0:
        return False
    
    return True


def printLists(documentDict):
    '''Print document lists
    @todo: getListContent
    '''
    if checkList(documentDict) is True:
        listElements = documentDict['content.xml'].getElementsByTagName ('text:list')
        for element in listElements:
            print 'Lista tehty', element.getAttribute('text:style-name'), '-tyylillä, lista:'
            listContent = element.getElementsByTagName ('text:list-item')
            for text in listContent:
                print "-", common_methods.getTextContent(text)
    
    
def getObjectPaths(documentDict):
    '''Get objects paths.
    Searches if document have an image.
    
    @return: object path list if founds an image if not return error
    '''
    objectPathList = []
    objectElements = documentDict['content.xml'].getElementsByTagName ('draw:object')
    if objectElements:
        for element in objectElements:
            objectPath = element.getAttribute ('xlink:href')
            objectPathList.append(objectPath)  
        return objectPathList 
    else:
        return False #error: there is no objects at all"
    
def _isStyleUsed(documentDict, styleName):
    '''Check if given style is used in document.
    
    @return: True if used, otherwise return False.
    '''
    usedStyleList = _getListOfUsedStyleNames (documentDict)
    if styleName in set(usedStyleList):
        return True
    return False

def getStyle(documentDict, styleName): 
    '''Get style defination attributes by given style name.
    parentStyleList is for executing the inheritation of styles.
    
    @return: style defination dictionary.
    
    Inheritation of styles:
      default paragraph -style-> standard-style -> style(Text body) -> P-style -> T-style
      
    XML (styles.xml):
    
    <style:style style:name="Standard" style:family="paragraph" style:class="text">
        <style:paragraph-properties fo:orphans="2" fo:widows="2" style:writing-mode="lr-tb"/>
        <style:text-properties style:use-window-font-color="true" style:font-name="Courier New" fo:font-size="10pt" fo:language="fi" fo:country="FI" style:font-name-asian="Times New Roman" style:font-size-asian="10pt" style:font-name-complex="Times New Roman" style:font-size-complex="10pt" style:language-complex="ar" style:country-complex="SA"/>
    </style:style>
    
    <style:style style:name="Text_20_body" style:display-name="Text body" style:family="paragraph" style:parent-style-name="Standard" style:class="text" style:master-page-name="">
        <style:paragraph-properties fo:margin-left="1cm" fo:margin-right="0cm" fo:margin-top="0.247cm" fo:margin-bottom="0.247cm" fo:text-indent="0cm" style:auto-text-indent="false" style:page-number="auto" fo:break-before="auto" fo:break-after="auto"/>
        <style:text-properties style:font-name="Tahoma"/>
    </style:style
    
    XML (content.xml):
    
    <style:style style:name="P2" style:family="paragraph" style:parent-style-name="Text_20_body">
        <style:paragraph-properties fo:text-align="start" style:justify-single-word="false"/>
    </style:style>
    
    '''       
    originalStyleName = styleName
    styleNameLower = styleName.lower()
    if string.find(styleNameLower, 'body') != -1:
        styleName = 'Text_20_body' 
    if _getStyleElementByStyleName(documentDict, styleName) is None:
        if _getStyleElementByDisplayName(documentDict, styleName) is None:
            return False
        else: styleName = _getStyleElementByDisplayName(documentDict, styleName).getAttribute ('style:name')
    
    if _isStyleUsed(documentDict, styleName) is False:
        return False
    
    defaultStyleElement = _getDefaultStyleElement(documentDict, 'paragraph')

    fontName = defaultStyleElement.getElementsByTagName('style:text-properties')[0].getAttribute('style:font-name')
    fontSize = defaultStyleElement.getElementsByTagName('style:text-properties')[0].getAttribute('fo:font-size')
    language = defaultStyleElement.getElementsByTagName('style:text-properties')[0].getAttribute('fo:language')
        
    
    styleAttributeDict = {
    'style:name':styleName,
    'style:parent-style-name':None,
    'fo:language':language,
    'style:font-name':fontName,
    'fo:font-size':fontSize,
    'fo:text-transform':None,
    'fo:text-indent':'0',
    'fo:line-height':'100%',
    'fo:margin-left':0,
    'fo:margin-right':0,
    'fo:margin-top':0,
    'fo:margin-bottom':0,
    'fo:keep-with-next':False,
    'fo:text-align': 'start',
    'fo:orphans':None,
    'fo:widows': None,
    'fo:font-style': False,
    'fo:font-weight':False}
    
    translateDict = {
    'style:name': 'styleName',
    'style:font-name':'fontName',
    'fo:font-size':'fontSize',
    'fo:text-transform':'transform',
    'fo:margin-left':'indentLeft',
    'fo:margin-right':'indentRight',
    'fo:text-indent':'indentFirstLine',
    'fo:line-height':'linespacing',
    'fo:margin-top':'spacingBefore',
    'fo:margin-bottom':'spacingAfter',
    'fo:keep-with-next':'keepWithNext',
    'fo:text-align': 'alignment',
    'fo:font-style': 'italic',
    'fo:font-weight':'bold',
    'widowControl':'widowControl'
    }
    styleDict = {}
    
    parentStyleList = _getParentStyleList (documentDict, styleName)
    
    #Get style attribute values
    while parentStyleList:
        styleAttributeDict['style:name'] = parentStyleList.pop()
        styleAttributeDict = _getStyleAttributes (documentDict, styleAttributeDict)
    
    #Conversions:   
    styleAttributeDict['fo:margin-top'] = conversions.convertCmOrInToPt(styleAttributeDict['fo:margin-top'])
    styleAttributeDict['fo:margin-bottom'] = conversions.convertCmOrInToPt(styleAttributeDict['fo:margin-bottom'])
    styleAttributeDict['style:name'] = originalStyleName #_getStyleDisplayNameByStyleName(documentDict, styleAttributeDict['style:name']).lower()
    styleAttributeDict['fo:line-height'] = conversions.convertPercentToDecimal(styleAttributeDict['fo:line-height'])
    styleAttributeDict['fo:font-size'] = str(round(float(styleAttributeDict['fo:font-size'].split('pt')[0]), 1))
    styleAttributeDict['fo:text-indent'] = conversions.convertCmOrInToString(styleAttributeDict['fo:text-indent'])
    styleAttributeDict['fo:margin-left'] = conversions.convertCmOrInToString(styleAttributeDict['fo:margin-left'])
    styleAttributeDict['fo:margin-right'] = conversions.convertCmOrInToString(styleAttributeDict['fo:margin-right'])
    if styleAttributeDict['fo:font-weight'] == 'bold': styleAttributeDict['fo:font-weight'] = True
    elif styleAttributeDict['fo:font-weight'] == 'normal': styleAttributeDict['fo:font-weight'] = False
    if styleAttributeDict['fo:font-style'] == 'italic': styleAttributeDict['fo:font-style'] = True
    elif styleAttributeDict['fo:font-style'] == 'normal': styleAttributeDict['fo:font-style'] = False
    if styleAttributeDict['fo:keep-with-next'] == 'always': styleAttributeDict['fo:keep-with-next'] = True
    if styleAttributeDict['fo:text-align'] == 'justify': styleAttributeDict['fo:text-align'] = 'both'
    elif styleAttributeDict['fo:text-align'] == 'start': styleAttributeDict['fo:text-align'] = 'left'
    elif styleAttributeDict['fo:text-align'] == 'end': styleAttributeDict['fo:text-align'] = 'right'
    
    
    if styleAttributeDict['fo:widows'] >= 2 and styleAttributeDict['fo:orphans'] >= 2:
        styleAttributeDict['widowControl'] = True #same in docx
    else:
        styleAttributeDict['widowControl'] = False #same in docx
    
    
    #Translate keys
    for key in translateDict.keys():
        styleDict[translateDict[key]] = styleAttributeDict[key]
        
    return styleDict 

def _getStyleAttributes (documentDict, styleAttributeList):
    '''Searches if style have wanted attribute if have then replace value.
    Style attribute list contains all relevant style information
    
    @return: list of styles attributes    
    '''
    styleElement = _getStyleElementByStyleName(documentDict, styleAttributeList['style:name'])
       
    if styleElement:
        paragraphPropertiesElement = styleElement.getElementsByTagName('style:paragraph-properties')
        textPropertiesElement = styleElement.getElementsByTagName('style:text-properties')
    
        for attribute in styleAttributeList.keys():
            if styleElement.hasAttribute (attribute): 
                styleAttributeList[attribute] = styleElement.getAttribute (attribute)
    
        if paragraphPropertiesElement:        
            for attribute in styleAttributeList.keys():            
                if paragraphPropertiesElement[0].hasAttribute (attribute):
                    styleAttributeList[attribute] = paragraphPropertiesElement[0].getAttribute (attribute)
    
        if textPropertiesElement:
            for attribute in styleAttributeList.keys():
                if textPropertiesElement[0].hasAttribute (attribute):
                    if attribute == 'fo:font-size' and textPropertiesElement[0].getAttribute(attribute).endswith('%'):
                        styleAttributeList[attribute] = str(int(styleAttributeList[attribute].split('pt')[0]) * int(textPropertiesElement[0].getAttribute(attribute).split('%')[0]) / 100)              
                    else:
                        styleAttributeList[attribute] = textPropertiesElement[0].getAttribute(attribute)
        
    return styleAttributeList

    
def _getParentStyleList (documentDict, styleName):
    '''Get patrent style list for given style name.
    Lists styles parent styles.
    
    @return: a list of parent styles (lists first entry is style itself)
    '''
    parentStyleList = [styleName]
       
    while _checkParentStyle(documentDict, styleName):
        styleName = _checkParentStyle(documentDict, styleName)
        parentStyleList.append(styleName)    
    
    return parentStyleList


def _checkParentStyle (documentDict, styleName):
    '''Check if style have a parent style
    
    @return: parent style name.
    '''
    try:
        return _getStyleElementByStyleName(documentDict, styleName).getAttribute('style:parent-style-name')
    except:
        return None
  
def checkEndnotesAndFootnotes(documentDict):
    '''Check end- and footnotes.
    
    @return: True if there is endnote or footnote in document, else return False.
    '''
    noteElements = documentDict['content.xml'].getElementsByTagName('text:note')
    if len(noteElements) == 0:
        return False
#        for element in noteElements:
#            if element.getAttribute ('text:note-class')=='endnote':
#                return True
#            elif element.getAttribute ('text:note-class')=='footnote':
#                return True
    return True

                
def checkImageCaptions(documentDict):
    '''Check image caption and reference.
    
    @return: True if document images have caption and reference, otherwise return False.
    '''
    caption = False
    reference = False
    imageElements = documentDict['content.xml'].getElementsByTagName ('draw:image')
    if imageElements:
        for element in imageElements:
            captionNode = element.parentNode.parentNode
            if len(common_methods.getTextContent(captionNode)) > 0:
                caption = True
            imagesReferenceElements = captionNode.getElementsByTagName('text:sequence')
            if imagesReferenceElements:
                reference = True
                
    if caption is False and reference is False:
        return False
    elif caption is False:
        return False #could replace with caption error message
    elif reference is False:
        return False #could replace with reference error message
    return True
     

def checkCoverPage(documentDict):
    '''Check that the front page is done correctly
    
    @return: the cover definitions in a dictionary.
     
    @keyword title: True if title in cover page is same that is in document meta.
    @keyword name: True if cover page contains same author name that is in document meta.
    @keyword email: True if cover page contains e-mail address.
    
    '''
    elementList = _getSectionElements(documentDict, 'cover')
    if elementList is None:
        return False
    meta = ooo_meta_inspector.getMeta(documentDict)
    cover = {
    'title':False,
    'name':False,
    'email' :False
    }
    for element in elementList:        
        if common_methods.checkIfEmailAddress (element):
            cover['email'] = True
        if common_methods.checkStringFromContent(element, meta['dc:creator']):
            cover['name'] = True
        if common_methods.checkStringFromContent(element, meta['dc:title']):
            cover['title'] = True
    return cover
                    


     
def getPageNumberFormatAndAuthor (documentDict, section):
    '''Get page number format and author name from document.
    
    @param section: 'cover', 'toc' or 'text'
    
    @return: dictionary
    '''
    sectionList = _getSectionBreakElements(documentDict)
    sectionBrakeElements = {'cover':sectionList[0], 'toc':sectionList[1], 'text':sectionList[2]}
    
    styleName = sectionBrakeElements[section].getAttribute('text:style-name')

    styleElement = _getStyleElementByStyleName(documentDict, styleName)
    
    masterPageStyleName = _getMasterPageStyleName(documentDict, styleElement)
    masterPageStyleElement = _getMasterPageStyleElement (documentDict, masterPageStyleName)
    authorAndNumberDict = getAuthorAndPageNumberFormat(documentDict, masterPageStyleElement)

    
    authorAndNumberDict['numStart'] = styleElement.getElementsByTagName('style:paragraph-properties')[0].getAttribute('style:page-number') 
    return authorAndNumberDict 


def checkHeadersAndFooters(documentDict):
    '''Check that the headers and footers of a document are made correctly.
    
    Assumes that the document has three sections:
        1. cover section
        2. table of contents section or toc section
        3. actual content section or text section

    @see checkSections - method must pass in order to run this method
    

    Places findings in the headerAndFooterDict as key-boolean pairs:
    'frontPage': was there headers or footers in the cover section.
    'tocPageNumbering': is there a page numbering in the toc section.
    'differentPageNumbering': is the page numbering different in the cover and text sections.
    'nameInToc': is the last modifiers name in toc section header or footer.
    'nameInText': is the last modifiers name in text section header or footer.
    'pageNumbering': is there a page numbering in the text section.
    'tocNumStart': does the toc section page numbering start at 1.
    'textNumStart': does the text section page numbering start at 1.
    '''
    headerAndFooterDict = {'frontPage': False, 'tocPageNumbering': False, 'differentPageNumbering': False,
            'nameInToc': False, 'nameInText': False, 'pageNumbering': False, 'tocNumStart': False,
            'textNumStart': False}
    
    cover = getPageNumberFormatAndAuthor (documentDict, 'cover')
    toc = getPageNumberFormatAndAuthor (documentDict, 'toc')
    text = getPageNumberFormatAndAuthor (documentDict, 'text')
    
    if cover['footerAuthor'] or cover['headerAuthor'] or cover['headerPageNumber'] or cover['footerPageNumber'] is not None:
        headerAndFooterDict ['frontPage'] = True
    else: headerAndFooterDict ['frontPage'] = False
        
    if toc['footerAuthor'] or toc['headerAuthor'] is not None:
        headerAndFooterDict ['nameInToc'] = True  
    if toc['headerPageNumber'] or toc['footerPageNumber'] is not None: 
        headerAndFooterDict['tocPageNumbering'] = True
        
    if text['footerAuthor'] or text['headerAuthor'] is not None:
        headerAndFooterDict ['nameInText'] = True 
    if text['headerPageNumber'] or text['footerPageNumber'] is not None:
        headerAndFooterDict['pageNumbering'] = True
    
    #TODO: tarkastus myös headerista (mitä jos on useita sivunumeroita???)
    if toc['footerPageNumber'] != text['footerPageNumber']:
        headerAndFooterDict['differentPageNumbering'] = True
    
    if toc['numStart'] == '1':
        headerAndFooterDict ['tocNumStart'] = True
        
    if text['numStart'] == '1':
        headerAndFooterDict ['textNumStart'] = True
    return headerAndFooterDict


def _getSectionElements (documentDict, section):
    '''Get elements of wanted section.
    Section break elements changes section.
    First list elements to cover-section.
    Second list elements to toc-section.
    And last list element to text-section.
    Document have to have atleast 3 sections.
    
    @return: section elements in list.
    '''
    sectionBreakElementList = _getSectionBreakElements(documentDict)
    sectionElements = {'cover':None, 'toc':None, 'text':None}
    sectionList = []
    officeBodyElement = documentDict['content.xml'].getElementsByTagName ('office:body')[0] #always exact 1 element
    documentElementList = officeBodyElement.firstChild.childNodes
    i = 0
    k = len(documentElementList)
    if len(sectionBreakElementList) < 3:
        return None
    while not documentElementList[i].isSameNode(sectionBreakElementList[1]):
        sectionList.append(documentElementList[i])
        i += 1
    sectionElements ['cover'] = sectionList 
    sectionList = []
    while not documentElementList[i].isSameNode(sectionBreakElementList[2]):
        sectionList.append(documentElementList[i])
        i += 1
    sectionElements ['toc'] = sectionList
    sectionList = []
    while i < k:
        sectionList.append(documentElementList[i])
        i += 1
    sectionElements ['text'] = sectionList        
    
    return sectionElements[section]    

    
def checkSections(documentDict, errorList):
    '''Check that document sections have made correctly.
    If amount of section breaks is not over 3 then return false.
    
    @return: True if sections are ok, return errorList if not ok.
    '''
    toc = False
    cover = True
    sections = len(_getSectionBreakElements (documentDict))
    if sections < 3:
        return False
    coverElements = _getSectionElements(documentDict, 'cover')
    for coverElement in coverElements:
        if coverElement.nodeName == 'text:table-of-content':
            cover = False
            errorList.append('cover')
            
    tocElements = _getSectionElements(documentDict, 'toc')
    for tocElement in tocElements:
        if tocElement.nodeName == 'text:table-of-content':
            toc = True
            break
    if toc is False:   
        errorList.append('toc')
        
    if toc and cover is True:
        return True
    else:
        return errorList
    

def _getMeta(documentDict):
    '''Get all meta information.
    
    @see: ooo_meta_inspector.getMeta
    
    @return: all meta in dictionary.    
    '''
    return ooo_meta_inspector.getMeta(documentDict)

def getMetaAuthor (documentDict):
    '''Get author, which have last modified the document.
    
    @return: last modified author.
    '''
    
    metaDict = ooo_meta_inspector.getMeta(documentDict)
    return metaDict ['dc:creator']

def getMetaTitle (documentDict):
    '''Get document title from meta.
    
    @return: title
    '''
    metaDict = ooo_meta_inspector.getMeta(documentDict)
    return metaDict ['dc:title']

def getMetaEdited (documentDict):
    '''Get last modified date and time from meta.
    
    @return: in ISO 8601 standard (yyyy-mm-ddThh:mm:ss)
    '''
    metaDict = ooo_meta_inspector.getMeta(documentDict)
    return metaDict ['dc:date']    

#FIXME: koko kappaleeseen käsin tehdyt muutokset tulevat P-tyyleihin (ei T), joten nämä jäävät huomioimatta/
#pitäisi tehdä tarkastus, jolla lailliset P-tyylit erotetaan laittomista
def checkStyleUsage (documentDict, errorIdsAndPositions):
    '''Goes through all elements in a document which have used style.
    Checks that elements are using correct styles (not Standard/Default style) and that no manual style definitions are made (like T1).
    
    '''
    
    illegalStyles = []
    for styleElement in _getListOfUsedStyleElements(documentDict):
        styleName = styleElement.getAttribute('text:style-name')
        if styleName[0] == 'T' and styleName[1].isdigit():
            illegalStyles.append(styleName)
        
        elif styleName[0] == 'P' and styleName[1].isdigit():
            styleElement = _getStyleElementByStyleName(documentDict, styleName)
            for child in styleElement.childNodes:
                if child.hasAttribute('fo:font-style')or child.hasAttribute('fo:text-align')or child.hasAttribute('fo:font-weight'):
                    illegalStyles.append(styleName)
                    
    for element in _getListOfUsedStyleElements(documentDict):
        if element.getAttribute('text:style-name') in set(illegalStyles):
            errorIdsAndPositions['manualChanges'].append(common_methods.getTextContent(element)[:30])
        elif element.getAttribute('text:style-name') == 'Standard':
            errorIdsAndPositions['styleNotUsed'].append(common_methods.getTextContent(element)[:30])
    
         
   
