#!/usr/bin/python
# -*- coding: UTF-8 -*-
#
#The MIT License
#
#Copyright (c) 2011
#
#Permission is hereby granted, free of charge, to any person obtaining a copy
#of this software and associated documentation files (the "Software"), to deal
#in the Software without restriction, including without limitation the rights
#to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#copies of the Software, and to permit persons to whom the Software is
#furnished to do so, subject to the following conditions:
#
#The above copyright notice and this permission notice shall be included in
#all copies or substantial portions of the Software.
#
#THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
#THE SOFTWARE.
#
#Authors:
#   Vili Auvinen (vili.k.auvinen@jyu.fi)
#   Olli Kauppinen (olli.kauppinen@jyu.fi)
#   Juho Tammela (juho.i.tammela@jyu.fi)

'''
This module makes the comparisons between the office document properties and requirements specified for each user.

@author: Vili Auvinen, Olli Kauppinen, Juho Tammela
'''

import docx_inspector, odt_inspector
import xml.dom.minidom


inspectorDict = {'odt': odt_inspector, 'docx': docx_inspector, 'docm': docx_inspector}
# @see: more attributes before inspect
#checkers = { 'TOC': word_processing.checkToc }

def makeDocumentDict(file, fileExtension):
    ''' Make a document dict which contains the XML files from a file by the given fileExtension.

    @see: _checkers

    @param fileExtension: docx/odt/odp etc.

    @return: document dict
    
    Code example:
        docXml = document['word/document.xml']
    '''

    document = {'fileExtension': fileExtension}
    
    for data in file.filelist:
        try:
            document[data.filename] = xml.dom.minidom.parseString(file.read(data.filename))
        except:
            document[data.filename] = file.read(data.filename)
    return document

def processRequirements(inspector, document, requirements, results):
    ''' Process requirements by looping through checkers dict which contains the method names described in the 
    XML requirement file. 
    
    @param inspector: the given inspector
    @param document: document dict which contains the XML files 
    @param requirements: Requirements object which contains the given requirements in the XML requirement file
    @param results: the given errors in a results dict 
    '''
    for requirement in requirements.requirements:
        _checkers[requirement.name](inspector, requirement, document, results)

def printResults(resultsDict):
    ''' For testing. '''
    print "\nERRORS:"
    for feed in resultsDict.keys():
        print feed.upper(),
        if resultsDict[feed]:
            print ''
            for i in resultsDict[feed]:
                print "  -", i
        else:
            print '...OK'
    
def checkBooleanRequirement(function, requirement, document, results):
    ''' Check boolean requirements. Inspector methods return a boolean.
    
    @param parameters: see processRequirements
    
    @return: True if expectedValue is returned from the inspector method, False otherwise. 
    '''
    expectedValue = requirement.expectedValue
    
    if str(function(document)) == str(expectedValue):
        return True
    else:
        results[requirement.category].append(requirement.getErrorMessage())
        return False
    
def checkDictRequirement(function, requirement, document, results):
    ''' Check dict requirement. Inspector methods return a dictionary of which the values are compared to the ones in
    XML requirement file.
    
    @param parameters: see processRequirements
    
    '''
    #FIXME: requirement's expectedValue is always a string and not the same type as inspector return value. 
    # Handle the types!!
    inspectorData = function(document)
    if inspectorData is False:
        results[requirement.category].append(requirement.getErrorMessage())
        return
    
    for key in requirement.expectedValue.keys():
        if str(inspectorData[key]) != str(requirement.expectedValue[key]):
            results[requirement.category].append(requirement.getErrorMessage(key))
        #elif function(document) is not 'True':
        #    results[requirement.category].append(requirement.getErrorMessage())
        
def checkRequirementEndNoteFootNote(inspector, requirement, document, results):
#    if checkBooleanRequirement(inspector.checkEndnotesAndFootnotes, requirement, document, results) is False:
#        results[requirement.category].append(requirement.getErrorMessage())
    checkBooleanRequirement(inspector.checkEndnotesAndFootnotes, requirement, document, results)
    
def checkRequirementSections(inspector, requirement, document, results):
    ''' Check requirement sections. The inspector method takes an empty list as an argument. If its length is not zero
    after the inspector method, it means that errors were appended. 
    
    @see: processRequirements for parameters
    
    @return: out from the method if inspectorData return False 
    '''
    
    errorList = []
    inspectorData = inspector.checkSections(document, errorList)
    
    if inspectorData is False and len(errorList) != 0:
        for element in errorList:
            results[requirement.category].append(requirement.getErrorMessage(element))
        else:
            results[requirement.category].append(requirement.getErrorMessage())
            
        return # can not continue checking if sections are wrong 
    elif inspectorData is not True and len(errorList) != 0: # if a list is returned
        # this if could be useless??? 
        for element in errorList:
            results[requirement.category].append(requirement.getErrorMessage(element))
        return


    #if inspectorData is True:
        # check different page numbering etc.
    
    #FIXME: not implemented
#    checkDictRequirement(inspector.checkSections, requirement, document, results)
    
def checkRequirementMargins(inspector, requirement, document, results):
    ''' Check requirement margins by calling the checkDictRequirement.
    
    @see: processRequirements for parameters and checkDictRequirement for the actual method.
    ''' 
    checkDictRequirement(inspector.getPageMarginals, requirement, document, results)

def checkRequirementPageSize(inspector, requirement, document, results):
    checkDictRequirement(inspector.getPageSize, requirement, document, results)
    
def checkRequirementCoverPage(inspector, requirement, document, results):
    checkDictRequirement(inspector.checkCoverPage, requirement, document, results)
    
def checkRequirementHeadingNumbering(inspector, requirement, document, results):
    ''' Check heading numbering requirement. 
    
    Error ids and positions are defined in a dict which is then used in the method 
    inspector.checkHeadingNumbering(document, errorIdsAndPositions). If the keys in 
    the dict are not None, errors have been appended. 
    
    @see: docx_inspector.checkHeadingNumbering(document, errorIdsAndPositions)
    @see: processRequirements for parameters
    '''
    
    expectedValue = requirement.expectedValue
    
    errorIdsAndPositions = { 'manualNumbering': None,
                             'styleNotUsed': None,
                             'differentNumbering': None,
                             'notMultilevel': None,
                             'outlineLvl': None,
                             'numStart': None,
                             'numWrong': None,
                             'numFormat': None,
                             'notSequential': None} # @see requirements.xml - <errorvalue>
    
    returnValue = inspector.checkHeadingNumbering(document, errorIdsAndPositions)

    if str(returnValue) != str(expectedValue):
        results[requirement.category].append(requirement.getErrorMessage()) #DEFAULT
    
    if len(errorIdsAndPositions.values()) != 0:
        for key in errorIdsAndPositions.keys():
            if errorIdsAndPositions[key] is not None:
                results[requirement.category].append(requirement.getErrorMessage(key) + ' ' + errorIdsAndPositions[key])
                # esim. - Seuraavan otsikkotyylin monitasoinen numerointi on väärin: Otsikko2
                    
def checkRequirementStyles(inspector, requirement, document, results):
    ''' Check style requirements. Compares the style requirements described in the XML file to document properties defined
    by the user. Appends a default error if inspector.getStyle returns False.
    
    @return: Nothing if inspector.getStyle returns False.
    
    @see: processRequirements for parameters
    '''
    
    inspectorData = inspector.getStyle(document, requirement.expectedValue['styleName'])
    
    if inspectorData is False:
        results[requirement.category].append(requirement.getErrorMessage())
        return
    
    styleName = requirement.expectedValue['styleName']
    styleFeedback = {styleName : []}
    
    for key in requirement.expectedValue.keys():
        if str(inspectorData[key]) != str(requirement.expectedValue[key]):
            styleFeedback[styleName].append(requirement.getErrorMessage(key))
    
    if len(styleFeedback[styleName]) > 0:
        results[requirement.category].append(styleFeedback)
        
def checkRequirementTOC(inspector, requirement, document, results):
    ''' Check if table of contents exists. If it does not exist, append a default error message.
    
    If it exists, check if the table of contents is correctly made. If not, append an error message. 
    
    @see: processRequirements for parameters
    '''
    
    if inspector.checkTOC(document) is True:
        if inspector.checkTocContent(document) is False:
#FIXME: is there a better way to get the error message than hard code?
            results[requirement.category].append(requirement.getErrorMessage('2'))
    else:
        results[requirement.category].append(requirement.getErrorMessage())
    
def checkRequirementImages(inspector, requirement, document, results):
    ''' Check if there are images in the document. Calls the checkBooleanRequirement function. 
    
    @see: checkBooleanRequirement(function, requirement, document, results)
    '''
    
    checkBooleanRequirement(inspector.checkImages, requirement, document, results)

def checkRequirementEmptyParagraphs(inspector, requirement, document, results):
    ''' Check empty paragraphs from document. Append an error if message if there are some to be found.
    
    @see: processRequirements for parameters
    '''
    
    expectedValue = requirement.expectedValue
    inspectorData = inspector.checkEmptyParagraphs(document)
    
    if str(inspectorData) == str(expectedValue):
        return
    else:
        results[requirement.category].append(requirement.getErrorMessage() + str(inspectorData))
        
def checkRequirementList(inspector, requirement, document, results):
    ''' Check if there are lists in the document. Calls the checkBooleanRequirement function.
    
    @see: processRequirements for parameters
    '''
    
    checkBooleanRequirement(inspector.checkList, requirement, document, results)
    
def checkRequirementStyleUsage(inspector, requirement, document, results):
    ''' Check style usage. An error dict (below) with two key-value pairs is used in the method inspector.checkStyleUsage.
    The inspector method returns the text paragraphs where manual changes have been made or style has not been used at all.
    
    errorIdsAndPositions = {'styleNotUsed': [], 'manualChanges': []} 
    
    @see: processRequirements for parameters
    
    '''
    errorIdsAndPositions = {'styleNotUsed': [], 'manualChanges': []} 
    # @see: requirements.xml - <errorvalue>
    
    
    result = inspector.checkStyleUsage(document, errorIdsAndPositions)
    styleNotUsedFeedback = {requirement.getErrorMessage('styleNotUsed') : []}
    manualChangesFeedback = {requirement.getErrorMessage('manualChanges') : []}
    if str(result) != str(requirement.expectedValue):

#        for key in errorIdsAndPositions.keys():
#            styleUsageFeedback[requirement.getErrorMessage(key)] = []
#            for item in errorIdsAndPositions[key]:
                #results[requirement.category].append(requirement.getErrorMessage(key) + unicode(item))
#                styleUsageFeedback[requirement.getErrorMessage(key)].append(unicode(item))
        for item in errorIdsAndPositions['styleNotUsed']:
            styleNotUsedFeedback[requirement.getErrorMessage('styleNotUsed')].append(unicode(item))
        for item in errorIdsAndPositions['manualChanges']:
            manualChangesFeedback[requirement.getErrorMessage('manualChanges')].append(unicode(item))
        
    #for key in styleUsageFeedback.keys():
    if len (styleNotUsedFeedback[requirement.getErrorMessage('styleNotUsed')]) > 0:
        results[requirement.category].append(styleNotUsedFeedback)
    if len (manualChangesFeedback[requirement.getErrorMessage('manualChanges')]) > 0:
        results[requirement.category].append(manualChangesFeedback)
            
def checkRequirementTabs(inspector, requirement, document, results):
    ''' Check if tabs have been used in the document. Do nothing if tabs are not found, otherwise append an error.
    
    @see: processRequirements for parameters
    '''
    
    expectedValue = requirement.expectedValue
    inspectorData = inspector.checkTabs(document)
    
    if str(inspectorData) == str(expectedValue):
        return
    else:
        results[requirement.category].append(requirement.getErrorMessage() + str(inspectorData))

def checkRequirementDoubleWhitespace(inspector, requirement, document, results):
    ''' Check if double whitespaces are found in the document. Do nothing if double whitespaces are not found, otherwise
    append an error.
    
    @see: processRequirements for parameters
    '''
    
    expectedValue = requirement.expectedValue
    inspectorData = inspector.checkDoubleWhitespaces(document)
    
    if str(inspectorData) == str(expectedValue):
        return
    else:
        results[requirement.category].append(requirement.getErrorMessage() + str(inspectorData))
        
def checkRequirementAsterisk(inspector, requirement, document, results):
    ''' Check if asterisks are found in the document. Do nothing if asterisks are not found, otherwise append an error.    
    A special print formatting is used here. Could be useful in other methods as well.
    
    @see: processRequirements for parameters
    '''
    
    expectedValue = requirement.expectedValue
    inspectorData = inspector.checkAsterisk(document)
    
    if str(inspectorData) == str(expectedValue):
        return
    else:
        results[requirement.category].append(requirement.getErrorMessage() % str(inspectorData))
        # % is used for print formatting
            
def checkRequirementImageCaptions(inspector, requirement, document, results):
    ''' Check if image captions are used.
    
    @see: checkBooleanRequirement
    @see: processRequirements for parameters
    '''
    
    checkBooleanRequirement(inspector.checkImageCaptions, requirement, document, results)

def checkRequirementHeadersAndFooters(inspector, requirement, document, results):
    ''' Check headers and footers requirement.
    
    This method can be run only if checkRequirementSections goes through. Hard to implement in a smart way. 

    @see: checkSections
    @see: processRequirements for parameters
    
    '''
    errorList = []
    inspectorSections = inspector.checkSections(document, errorList)
    if inspectorSections is False: 
        results[requirement.category].append(requirement.getErrorMessage())
        return
    
    elif inspectorSections is not True and len(inspectorSections) != 0: # jos palauttaa listan
        #------------------------------------- for element in inspectorSections:
        #results[requirement.category].append(requirement.getErrorMessage())
        return
    
    inspectorData = inspector.checkHeadersAndFooters(document)
        
    for key in requirement.expectedValue.keys():
        if str(inspectorData[key]) == str(requirement.expectedValue[key]):
            continue
        else: results[requirement.category].append(requirement.getErrorMessage(key))

def checkRequirementIndex(inspector, requirement, document, results):
    ''' Check if index is found in the document. If index is correctly made, checks the index content.
    
    @see: processRequirements for parameters
    
    '''

    #FIXME: error codes are hard coded in the inspector and in the requirements XML file.
    result = inspector.checkIndex(document)
    if str(result) != str(requirement.expectedValue):
        results[requirement.category].append(requirement.getErrorMessage(result))
        return
    
    result = inspector.checkIndexContent(document)
    if str(result) != str(requirement.expectedValue):
        results[requirement.category].append(requirement.getErrorMessage(result))
        
# This module processes requirements by looping through checkers dict (below) which contains the method 
# names described in the XML requirement file. 
_checkers = { 'Styles': checkRequirementStyles,
              'TOC': checkRequirementTOC,
              'Image' : checkRequirementImages,
              'CoverPage': checkRequirementCoverPage,
              'PageSize' : checkRequirementPageSize,
              'Margins' : checkRequirementMargins,
              'EndNoteFootNote' : checkRequirementEndNoteFootNote,
              'HeadingNumbering': checkRequirementHeadingNumbering,
              'EmptyParagraphs' : checkRequirementEmptyParagraphs,
              'CheckSections': checkRequirementSections,
              'List' : checkRequirementList,
              'StyleUsage' : checkRequirementStyleUsage,
              'Tabs': checkRequirementTabs,
              'DoubleWhitespaces': checkRequirementDoubleWhitespace,
              'ImageCaptions' : checkRequirementImageCaptions,
              'Asterisk': checkRequirementAsterisk,
              'Index': checkRequirementIndex,
              'CheckHeadersAndFooters': checkRequirementHeadersAndFooters
             }

def inspect(documentFile, requirements, fileExtension):
    ''' Inspect a document by the given file extension which is either odt or docx. 
    
    @param zipFile: docx or odt
    @param requirements: the requirements specified in the XML requirement file
    @param fileExtension: docx or odt
    '''
    
    results = {'common':[], 'structure':[], 'content':[], 'styles':[], 'numbers':[], 'formatting':[], 'objects':[]}
    document = makeDocumentDict(documentFile, fileExtension)
    inspector = inspectorDict[document['fileExtension']]
    
    #FIXME: hard coded for gui testing:
    #if requirements == "":
    #    reqFilename = urlopen('http://sovellusprojektit.it.jyu.fi/parsi/sovellus/sampleFiles/xml/requirements_test1.xml')

    #    reqDom = xml.dom.minidom.parse(reqFilename)
    #    requirements = Requirements(reqDom)
    
    processRequirements(inspector, document, requirements, results)
    
    return results

if __name__ == '__main__':
    ''' For testing. '''
#    inspect("foo", "vaatimukset.xml", "foobar")

    #filename = 'sampleFiles/docx/sivunumerointi_oikein_hdr_ftr.docx'
    #filename = 'sampleFiles/docx/tekstinkasittely.docx'
    #filename = 'sampleFiles/docx/teemakokeilu.docx'
    #filename = 'sampleFiles/docx/docx_demo.docx'
    #filename = 'sampleFiles/docx/otsikko_numerointi_ei_otsikko1.docx'
    #filename = 'sampleFiles/docx/docx_demo_ascii.docx'
    #filename = 'sampleFiles/docx/otsikko2ja4_numerointi.docx'
    #filename ='sampleFiles/odt/Yliopisto_opiskelu.odt'
    #filename ='sampleFiles/odt/toc_handmade.odt'
    
    #filename = 'sampleFiles/docx/index_lorem1.docx'
    #filename = 'sampleFiles/docx/index_lorem2.docx'
    #filename = 'sampleFiles/docx/hakemisto_kasin.docx'
    #filename = 'sampleFiles/docx/index_lorem2_whitespace_asterix.docx'
    filename = 'sampleFiles/docx/Teija_Holtta.docx' # asetus19.xml, see below
    #filename = 'sampleFiles/docx/Harjoitustyo_elina_asikainen.docx'
    #filename = 'sampleFiles/docx/HarjoitustyoD.docx'
    #filename = 'sampleFiles/docx/Harjoitustyo.docx'
    
    reqFilename = 'sampleFiles/xml/requirements_test1.xml'
    #zipFile = zipfile.ZipFile(filename)
    printResults(inspect(filename, reqFilename, "docx"))
